#include "DrawTool.h"

#define _USE_MATH_DEFINES
#include <math.h>
#include <time.h>
#include <list>

#include "PolyDraw.h"

DrawTool::DrawTool(PolyDraw *pd) : Tool(pd),down1(false),down2(false)
{
	draw->polygons.splice(draw->polygons.end(),draw->selected_polygons);
}

DrawTool::~DrawTool()
{
}

bool DrawTool::on_mouse_down(GdkEventButton *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	srand ( time(NULL) );
	if(event->button == 1)
	{
		down1 = true;
		Polygon *p = get_round_polygon(30,100);
		p->move(Vertex(x,-y));
		subtractPolygonFromDrawArea(p);
		draw->polygons.push_back(p);
	}
	else if(event->button == 3)
	{
		down2 = true;
		Polygon *p = get_round_polygon(30,100);
		p->move(Vertex(x,-y));
		subtractPolygonFromDrawArea(p);
	}
	return true;
}

bool DrawTool::on_mouse_up(GdkEventButton *event)
{
	if(event->button == 1)
	{
		down1 = false;
	}
	else if(event->button == 3)
	{
		down2 = false;
	}
	return true;
}

bool DrawTool::on_mouse_move(GdkEventMotion *event)
{
	int x = (int) event->x;
	int y = (int) event->y;

	if(down1)
	{
		Polygon *p = get_round_polygon(30,100);
		p->move(Vertex(x,-y));
		subtractPolygonFromDrawArea(p);
		draw->polygons.push_back(p);
	}
	else if(down2)
	{
		Polygon *p = get_round_polygon(30,100);
		p->move(Vertex(x,-y));
		subtractPolygonFromDrawArea(p);
	}
	else
	{
		return false;
	}
	return true;
}

bool DrawTool::redraw(Cairo::RefPtr<Cairo::Context>& cr)
{
	return false;
}

Polygon* DrawTool::get_round_polygon(int size, int smoothness)
{
	/*
	So what is this? It's basically a circle with the radius 'size'. The radius
	is adjusted by two sin/cos waves. The random lies in the phase of the sin/cos
	*/

	if(smoothness > 100) smoothness = 100;
	if(smoothness < 0) smoothness = 0;
	if(size <= 0) size = 1;

	std::list<Vertex> vertices;

	// line count of the polygon
	int count = size/2 + 6;

	// varying offset of the sin/cos waves
	double begin = 2*M_PI*(rand()%1000)/1000;
	double begin2 = 2*M_PI*(rand()%1000)/1000;

	// parameters:
	double anticircle = 3+pow((double)smoothness/16.0,2.0);	// the bigger the factor, the smaller the divergence from a circle
	int random = 200-smoothness;	// the bigger the factor the more random is the divergence from the circle

	for(int i=0; i < count; ++i)
	{
		double angle = 2*M_PI*i/count;

		int currsize = size;
		currsize += (int) (sin(angle*3 + begin + rand()%random/100.0) * size/anticircle);		// +sin
		currsize += (int) (cos(angle*5 + begin2 + rand()%random/100.0) * size/anticircle/2);	// +cos

		vertices.push_back(Vertex( (int) (sin(angle)*currsize) , (int) (-cos(angle)*currsize) ));
	}

	return new Polygon(vertices);
}

void DrawTool::subtractPolygonFromDrawArea(Polygon *newpoly)
{
		// first subtract
		std::list<Polygon *> clipResult;
		for(std::list<Polygon *>::iterator pit = draw->polygons.begin(); pit != draw->polygons.end();)
		{
			Clip_result cr = (*pit)->clip(*newpoly,Difference,clipResult);
			// hit it? delete it and add result polygons to list of polygons
			if(cr != SubjectPolygon)
			{
				Polygon *toDelete = *pit;
				pit = draw->polygons.erase(pit);
				delete toDelete;
				if(draw->polygons.empty()) break;
			}
			else ++pit;
		}
		draw->polygons.splice(draw->polygons.end(),clipResult);
}