#include "GluePolygonTool.h"

#include <list>

#include "Polygon.h"
#include "PolyDraw.h"

GluePolygonTool::~GluePolygonTool()
{
}

bool GluePolygonTool::on_mouse_down(GdkEventButton *event)
{
	if(event->button == 1)
	{
		int x = (int) event->x;
		int y = (int) event->y;
		selectPolygonAtPoint(Vertex(x,-y));
		down = true;
	}
	return true;
}

bool GluePolygonTool::on_mouse_up(GdkEventButton *event)
{
	if(event->button == 1)
	{
		down = false;
	}
	return false;
}

bool GluePolygonTool::on_mouse_move(GdkEventMotion *event)
{
	if(!down) return false;

	int x = (int) event->x;
	int y = (int) event->y;

	// if new polygons have been added, try to glue them...
	if(selectPolygonAtPoint(Vertex(x,-y)))
		gluePolygons();

	return true;
}

bool GluePolygonTool::redraw(Cairo::RefPtr<Cairo::Context>& cr)
{
	return false;
}

bool GluePolygonTool::selectPolygonAtPoint(const Vertex& point) const
{
	bool ok = false;
	// find polygon to select
	for(std::list<Polygon *>::iterator it = draw->polygons.begin(); it != draw->polygons.end(); ++it)
	{
		if((*it)->contains(point))
		{
			draw->selected_polygons.push_back(*it);
			draw->polygons.erase(it);
			ok = true;
			break;
		}
	}
	return ok;
}

void GluePolygonTool::gluePolygons() const
{
	for(std::list<Polygon *>::iterator it = draw->selected_polygons.begin(); it != draw->selected_polygons.end(); )
	{
		bool clipped = false;
		for(std::list<Polygon *>::iterator jt = draw->selected_polygons.begin(); jt != draw->selected_polygons.end(); ++jt)
		{
			if(it == jt) continue;
			std::list<Polygon *> clipResult;
			Clip_result cr = (*it)->clip(**jt,Union,clipResult);
			if(cr == Default)
			{
				if(clipResult.size() <= 1)
				{
					draw->selected_polygons.splice(draw->selected_polygons.end(),clipResult);
					delete *it;
					delete *jt;
					draw->selected_polygons.erase(jt);
					it = draw->selected_polygons.erase(it);
					clipped = true;
					break;
				}
				// we have no gain from that? leave it...
				else
				{
					// clear clip result
					for(std::list<Polygon *>::iterator clear_it = clipResult.begin(); clear_it != clipResult.end(); ++clear_it)
					{
						delete *clear_it;
					}
					clipResult.clear();
				}
			}
		}
		if(!clipped) ++it;
	}
}