/*
 * 2D Polygon Landscape
 *
 * Copyright (c) 2011  Tobias Zwick
 */

#include "Intersection.h"

#include "Line.h"
#include "Vertex.h"

#include <cstdlib>

/* ------------- Intersection ----------- */

Intersection::Intersection()
{
	line = NULL;
	point = NULL;
	thisParam = 0;
	otherParam = 0;
	exact = true;
}

Intersection::~Intersection()
{
	delete line;
	delete point;
}

Intersection::Intersection(const Intersection& in)
{
	line = (!in.line ? NULL : new Line(*in.line));
	point = (!in.point ? NULL : new Vertex(*in.point));
	thisNormal = in.thisNormal;
	otherNormal = in.otherNormal;
	thisParam = in.thisParam;
	otherParam = in.otherParam;
	exact = in.exact;
}

Intersection& Intersection::operator=(const Intersection& p)
{
	delete line;
	delete point;
	line = (!p.line ? NULL : new Line(*p.line));
	point = (!p.point ? NULL : new Vertex(*p.point));
	thisNormal = p.thisNormal;
	otherNormal = p.otherNormal;
	thisParam = p.thisParam;
	otherParam = p.otherParam;
	exact = p.exact;
	return *this;
}

void Intersection::clear()
{
	delete line;
	delete point;
	line = NULL;
	point = NULL;
}

bool Intersection::operator==(const Intersection& p) const
{
	if(!line) {
		if(p.line) return false;
	} else {
		if(!p.line) return false;
	}
	if(!point) {
		if(p.point) return false;
	} else {
		if(!p.point) return false;
	}

	if(thisNormal != p.thisNormal) return false;
	if(otherNormal == p.otherNormal) return false;
	if(*line != *(p.line)) return false;
	if(*point != *(p.point)) return false;
	if(thisParam != p.thisParam) return false;
	if(otherParam != p.otherParam) return false;
	if(exact != p.exact) return false;

	return true;
}

bool Intersection::isLineIntersection() const
{
	return line != NULL;
}