/*
 * 2D Polygon Landscape
 *
 * Copyright (c) 2011  Tobias Zwick
 */

#ifndef _INTERSECTION_H_
#define _INTERSECTION_H_

#include "Vertex.h"

class Line;

/* Intersection

	 This struct represents an intersection between lines,
	 It can consist of a point or a line. Additionally, the
	 position at which the intersection was found on the two
	 involved lines are stored in thisParam and otherParam
	 (ranging from 0..1) as well as the normals. In the field
	 exact, it is stored whether the point intersection found
	 is created at the exact position calculated or had to be
	 rounded.
*/
class Intersection
{
	public:
		Intersection();
		~Intersection();
		void clear();
		Intersection(const Intersection& in);
		Intersection& operator=(const Intersection& p);
		bool operator==(const Intersection& p) const;
		bool operator!=(const Intersection& p) const;
		// intersection data
		Line *line;
		Vertex *point;
		// parameter value where the intersection occured
		double thisParam;
		double otherParam;
		// for convenience, the normals of the intersection
		// have already been calculated
		Vertex thisNormal;
		Vertex otherNormal;
		// whether the intersection data is exact or has been rounded
		bool exact;
		// whether pointer to line != NULL
		bool isLineIntersection() const;
};

#endif