/*
 * 2D Polygon Landscape
 *
 * Copyright (c) 2011  Tobias Zwick
 */

#include <cmath>
#include "Vertex.h"

inline int myround(double x) { return (int)(x<0?ceil(x-0.5):floor(x+0.5)); }

/* --------------------- Vertex 2D ----------------------*/

Vertex::Vertex() : _x(0), _y(0)
{

}


Vertex::Vertex(const Vertex& v) : _x(v.x()), _y(v.y())
{

}

Vertex::Vertex(int xpos, int ypos) : _x(xpos), _y(ypos)
{

}

bool Vertex::operator==(const Vertex& v1) const
{
	return (v1.x() == _x && v1.y() == _y);
}

bool Vertex::operator!=(const Vertex& v1) const
{
	return !(v1.x() == _x && v1.y() == _y);
}

Vertex& Vertex::operator*=(const int v2)
{
	_x *= v2;
	_y *= v2;
	return *this;
}

Vertex& Vertex::operator/=(const int v2)
{
	_x /= v2;
	_y /= v2;
	return *this;
}

Vertex& Vertex::operator*=(const double v2)
{
	_x = int(_x*v2);
	_y = int(_y*v2);
	return *this;
}

Vertex& Vertex::operator/=(const double v2)
{
	_x = int(x()/v2);
	_y = int(y()/v2);
	return *this;
}

Vertex& Vertex::operator+=(const Vertex& v2)
{
	_x += v2.x();
	_y += v2.y();
	return *this;
}

Vertex& Vertex::operator-=(const Vertex& v2)
{
	_x -= v2.x();
	_y -= v2.y();
	return *this;
}

Vertex Vertex::operator+(const Vertex& v2) const
{
	return Vertex(_x + v2.x(), _y + v2.y());
}

Vertex Vertex::operator-(const Vertex& v2) const
{
	return Vertex(_x - v2.x(), _y - v2.y());
}

Vertex Vertex::operator*(const int v) const
{
	return Vertex(_x * v, _y * v);
}

Vertex Vertex::operator/(const int v) const
{
	return Vertex(_x / v, _y / v);
}

Vertex Vertex::operator*(const double v) const
{
	return Vertex(  myround(_x * v), myround(_y * v));
}

Vertex Vertex::operator/(const double v) const
{
	return Vertex(myround(_x / v), myround(_y / v));
}

int Vertex::operator*(const Vertex& v) const
{
	return _x*v.x() + _y*v.y();
}

Vertex Vertex::operator+() const
{
	return Vertex(*this);
}

Vertex Vertex::operator-() const
{
	return Vertex(-_x,-_y);
}
